#ifndef BIGNAT_H
#define BIGNAT_H

#include <stl.h>
#include <iostream.h>


// There are three different usages of const here. For example in 
//   const Bignat &operator+ ( const Bignat &) const;
//    1                         2               3
// What do they mean ?

 
// We use |b| for the interpretation (as a natural number)
// of b. We use 'me' for *this. (|me| is the interpretation of this object as
// a natural number. We use 'ret' to denote the return result



class Bignat
{
 friend ostream &operator<< ( ostream &, const Bignat &);
 friend istream &operator>> ( istream &, Bignat &);
// For input and output 

 public:
   Bignat ( int i = 0 );
   // Pre: i = X, Post: |me| = X
   Bignat ( const Bignat &b);
   // Pre: |b| = X, Post: |me| = X 
   ~Bignat ( void );
   // destructor
   

   const Bignat & operator= ( const Bignat &b);
   // Pre: |b| = X , post: |me| = X

   const Bignat &operator+ ( const Bignat &b) const;
   // Pre: |me| + |b| = X, Post: |ret| = X

   const Bignat &operator* ( const Bignat &b) const;
   // Pre: |me| * |b| = X, Post: |ret| = X

   const Bignat &operator- ( const Bignat &b) const;
   // Pre: |me| - |b| = X and X >= 0, Post: |ret| = X

   int operator< ( const Bignat &b) const;
   // Pre: (|me| < |b|) = X, Post: ret = X

   int operator== ( const Bignat &b) const; 
   // Pre: (|me| = |b|) = X, Post: ret = X

 private:
   void ten_times ( void );
   void n_times ( int );
   deque <int> r;

};



#endif
