
/*
	File: String.h
	Author: Oscar Nierstrasz 30.11.95
*/

#ifndef STRING_H
#define STRING_H

#include <iostream.h>
#include <exception.h>
#include <string.h>

class String
{
	friend ostream& operator<<(ostream&, const String&);
	// also need:
	// friend istream& operator>>(istream&, String&);

  public:
	String(void);					// default constructor:
	~String (void);					// destructor:
	String (const String& copy);		// copy constructor:

	String (const char*s);			// char* constructor
	// String(int);
	
	String& operator= (const String&);	// assignment

	// NB: "inline" keyword is optional here ...
	inline int strlen (void) const { return ::strlen(_s); }
	// Why is the const declaration needed?
	// (Otherwise not allowed to take strlen of a const String.)
	// Which other ops should be declared const?
	
	char& operator[] (const int n);
	String& operator+= (const String&);	// concatenation

	// comparisons: ==, <, >

	int getline (istream&);

  private:
	// invariant:
	// _s points to a null-terminated string on the heap
	char *_s;
	
	void become (const char*);		// internal copy function
	void grow (int);				// helper for getline()
};

// concatenation -- makes a new String
String operator+ (const String& s1,const String& s2);
// exercise -- redefine as a member function

// Instead of making this function a friend, we just make
// it an inline function that calls the privileged getline().
inline
istream& operator>> (istream& inStream, String& s)
{
	s.getline (inStream);
	return inStream;
	// what happens upon EOF?!
}

#endif // STRING_H

