
/*
	File: BoardGame.h
	Author: Oscar Nierstrasz 18.3.96

	Generic interface for simple board games with two players.

	Invariants:
	- initially winner is nobody
	- game is over when all squares are occupied,
	  or there is a winner
	- turn is X or O
	- initially it is the turn of X
	- X and O swap turns (turn never equals previous turn)

    Variable:
  	- rules for winning
  	- what moves are valid
  	- side effects of moves
	- number of squares
*/


#ifndef BOARDGAME_H
#define BOARDGAME_H

#include <iostream.h>		// Declare ostream

#ifdef __GNUC__
#include "/opt/SUNWspro/SC3.0.1/include/CC/exception.h"
#else
#include <exception.h>
#endif

#define DEBUG
#include "Debug.h"

#include "String.h"

class BoardGame {
  public :
	BoardGame (int rows = 8, int cols = 8)		// constructor
		throw(xmsg);

	// Destructor must be virtual if client doesn't
	// know class when deleting ...
	// Pure virtual since abstract class ...
	virtual ~BoardGame (void) = 0;

	// do we need copy constructor?
	// assignment operator?

	int notover (void);			// true if game is not over
	const char *winner (void);	// Winner is "X", "O" or "Nobody"
	char turn (void);			// Whose turn is it?

	void play (String& move)		// Current player marks a square
		throw (xmsg);			// Invalid move raises exception

	void print(void);			// Pretty-print the current state

  protected :
	// Symbolic names for players -- also used to index char []
	enum Player  {
		nobody	= 0,
		X		= 1,
		O		= 2
	};

	// Protected state:
	const int rows, cols;		// never changed
	Player _winner;				// Initially nobody
	Player _turn;				// Initially X
	int squaresLeft;			// Initially rows*cols
	Player ** square;			// r*c array Initially nobody

	// Char and string names of players.
	// Must be initialized outside of class [ARM p. 290]
	static const char player[3];
	static const char * winners [3];

	// Private helper functions for play and print:
	virtual void makeMove (int row, int col)
		throw (xmsg);				// mark a square
	virtual void checkWinner (int row, int col) = 0;	// Is there a winner?

	int inRange (int row, int col);	// (row,col) are in range
};

ostream& operator<< (ostream&, BoardGame&);

#endif // BOARDGAME_H
