
// File: Jumble.C
// Author: Oscar Nierstrasz 18.5.96
// Solve the Jumble puzzle words with STL

#include <iostream.h>
#include <fstream.h>

#include <algo.h>
#include <multimap.h>

// #define DEBUG

#include "String.h"
#include "Debug.h"

const char* const wordFile = "words"; // File containing word list

typedef multimap<String, String, less<String> > Dict;
typedef Dict::value_type StringPair;

void unJumble(void);
String makeKey(const String& word);
void lookup (const String& word, Dict& dict);

int main(void)
{
	unJumble();
	return 0;
}

void loadDict(Dict& dict);

class printWord {
	public:
		void operator() (StringPair& item) {
			cout << item.second << endl;
		}
};

void unJumble(void)
{
	String word, key;
	Dict dict;
	pair<Dict::iterator, Dict::iterator> range;

	loadDict(dict);

	while (true) {
		cout << "Unjumble: " << flush;
		cin >> word;

		if (word.strlen() == 0) {
			cout << "Bye!" << endl;
			return;
		}

		key = makeKey(word);
		range = dict.equal_range(key);

		if (range.first == range.second) {
			cout << "Can't unjumble " << word << endl;
		} else {
			cout << "Found:" << endl;
			for_each(range.first, range.second, printWord());
		}
	}
}

void loadDict(Dict& dict)
{
	String word;
	ifstream wordStream; // (wordFile);

	wordStream.open(wordFile);
	if (!wordStream) {
		cerr << "Invalid wordStream" << endl;
		return;
	}

#ifdef DEBUG
	int maxWords, count = 0;
	cout << "Max words to read: " << flush;
	cin >> maxWords;
	cin >> word; // flush
#endif // DEBUG

	while (word.getline(wordStream)) {
		dict.insert(StringPair(makeKey(word), word));
#ifdef DEBUG
		if ((++count)%100 == 0) {
			debug << "Entered " << count << " words: <"
				<< makeKey(word) << "," << word << ">" << endl;
		}
		if (count > maxWords) {
			cout << "Stopping at " << maxWords << " words" << endl;
			return;
		}
#endif // DEBUG
	}
}

String makeKey(const String& word)
{
	String key = word;
	sort(key.begin(), key.end());
	return(key);
}

