#include "Bignat.h"
#include <stl.h>
#include <string.h>
#include <iostream.h>
//#include <assert.h>

/* Numbers are represented as a sequence of digits, stored in deque r.
   (114) = 1 , 1 , 4 
   The digits are integers between 0 and 9
   The only time a sequence can start with 0 is when it is a 
   representation of 0. 

   We will use |b| to denote the representation of b in comments.
   We will use me to denote the representation of (*this)
   We will use ret to denote a returned value

   We will use capitalized variables (X, Y) to denote logical constants.
   (  i = X means: the value of i is equal to a constant X)

*/





Bignat::Bignat ( int i )
/*  Pre: 0 <= i and i  = X
    Post: |me| = X 
*/
{
  r.push_front(i % 10 ); // Enter the least significant digit.
  while ( (i / 10) > 0 ) // Are there any digits left ?
     {i = i / 10;        // Least significant digit in i is not inserted yet.
      r.push_front ( i % 10 ) ;
     }
}


  

Bignat::Bignat ( const Bignat &b) 
/* Pre: |b| = X
  Post: |me| = X
*/
{
  deque<int>::const_iterator i;
  for (i=b.r.begin();i != b.r.end();i++)  // Iterator i iterates throught the
                                          // deque. We have to use push_back.
                                          // Why ? 
      r.push_back ( *i);

}

Bignat::~Bignat ( void )
{ // deque is destructed automatically. Add the destructor as a
  // guide to the g++ compiler
 ;
}


   
const Bignat& Bignat::operator= ( const Bignat &b)
// Pre: |b| = X
//Post: |me| = X
{ // A different approach this time, using 
  // deque assignment operator

  if (this == &b) // Are we trying to assign ourself to ourself ? 
               return (*this); // Do nothing and return ourself.
   

  r = b.r;   // Use deque assignment operator

 return (*this);

}

const Bignat& Bignat::operator+ ( const Bignat &b) const
// Pre: |b| + |me| = X
// Post: |ret| = X
{// Using reverse iterators. Add (starting at least signifcant digit)
 // the digits, using carry

 
 deque <int>::const_reverse_iterator i; // const_iterator means that the 
 deque <int>::const_reverse_iterator j; // iterator is read only

 deque <int> result; // To store the result

 int carry = 0; // Carry holds the value that is carried over. 
                // (at most 1, at least 0)

 for (i = b.r.rbegin(), j = r.rbegin(); // Start at the reversed begin (end)
                                        // of our deque, and the one in b
       (i!= b.r.rend() || j!=r.rend()); ) // End when both deques are 
                                          // exhausted
     { int z = carry; // z will hold the temporary result of adding the 
                      // two current digits. 
       if (i !=b.r.rend()) {z = z + (*i); // Add the b digit if it exists
                            i++; // Go to the next (iterate) digit
                           }
       if (j !=r.rend()) { z = z + (*j); // Add our digit ....
                           j++; //iterate
                         }
       result.push_front (z % 10); /* Push the digit in result. We push it
                                      on the front, because we started at the
                                      least significant digits. This means that
                                      most significant digit so far is the  
                                      current one, and needs to go up front 
                                   */
     

       carry = z / 10; // Recalculate carry     
    } // End of loop

 if (carry != 0) result.push_front ( carry );
         // If there's still a carry, we need to push it up front. If not, we
         // are not allowed to (invariant)

 Bignat *c; // We use a pointer because we want to return a reference to 
            // a bignat. If we were to use Bignat c, and return c, we would
            // have a problem. Why ? 
 c = new Bignat; 
 c->r = result; // c->r is the same as (*c).r
 return (*c);
}

const Bignat &Bignat::operator- ( const Bignat &b) const 
// Pre: |me| - |b| = X and X >= 0
// Post: |ret| = X

{ // Only defined if the interpretaton of b is less than (or equal to)
  // the interpretation of a

 deque <int>::const_reverse_iterator i;
 deque <int>::const_reverse_iterator j;

 deque <int > result;

 int carry = 0; // Now carry is one if we 'borrowed' from the higher
                // significant digit. (for example 11 - 9 )

 for (i = b.r.rbegin(), j = r.rbegin(); // Start at the end
       (j!=r.rend()); j++) // b is less than me. So me holds at least
                           // as many entries as b
     { int z = carry; // z holds temporary result

       z =  (*j) - z; // take digit, substract carry
       if (i !=b.r.rend()) {z = z - (*i);  // substract digit if it exists.
                            i++; //iterate
                           }
       
       // Are we less than zero now ? Add 10 and set carry. Else carry is 0
       if (z < 0) { z = z + 10; carry = 1; }
           else      carry = 0;
   
       result.push_front ( z ); // Note that z will be >= 0 and < 10. Why ?

    }
 // It could be that result has leading zero's. Remove them unless there
 // is only one digit left.
 while  ( (*result.begin()) == 0 && (result.begin()+1 != result.end()) )
        result.pop_front();

 Bignat *c;
 c = new Bignat;
 c->r = result;


 return (*c);

}
 
int Bignat::operator< ( const Bignat &b ) const
// Pre: (|me| < |b|) = X
//Post: ret = X
  
{// Is the interpretation of this less than the interpretation of b ?
 if (r.size() < b.r.size()) return 1; // Longer is bigger because of 
 if (b.r.size() < r.size()) return 0; // invariant

 // deques are of the same size. We can use lexicographical compare to find
 // the greatest. (lexicographical compare is the same as dictionary
 // compare ) lexicographical compare is a generic algorithm provided
 // by STL
 
 return lexicographical_compare ( r.begin(), r.end(), b.r.begin(), b.r.end() );
}


int Bignat::operator== ( const Bignat &b) const
// Pre: (|me| = |b|) = X
// Post: |ret| = X 
{
  if (b.r == r)       return 1; // Using comparison on deque
  return 0;
 
// Note that we need the invariant to be able to use this
// implementation
};

void Bignat::ten_times ( void )
// This is a function needed in operator* and not to be exported.
// Pre: |me| = X
//Post: |me| = 10*X
{ // Multiplying by 10 is the same as adding a zero at the end
  // But (!) we must maintain the invariant.
 if ((r.begin()+1 == r.end()) && (*r.begin() == 0)) return; //to maintain inv.
 r.push_back ( 0 );
}
  


void Bignat::n_times ( int n )
// This is a function needed in operator* and not to be exported.
// Pre: |me| = X and n = N and 0 <= N and N < 10
//Post: |me| = N*X
{ n = n % 10; // Should not be necessary, but you never know

  if ( (r.begin()+1 == r.end()) && (*r.begin() == 0) ) return ;// n*0 is 0
  if (n == 0) { r.erase(r.begin(),r.end()); // erase the deque
                r.push_back(0); // Put one 0 in.
              }
      else    { deque <int> b;
                deque <int>:: reverse_iterator i; // Start at the back
                int carry = 0; 
                for (i = r.rbegin();i != r.rend(); i++) 
                    { b.push_front( ((*i)*n + carry) % 10 );
                      carry =  ((*i)*n + carry) / 10 ; 
                    }
                if (carry != 0 ) b.push_front ( carry % 10); 
                r = b; // assign the deque to r;             
              
               
               }
}
 

const Bignat &Bignat::operator* ( const Bignat &b) const
// Pre: |b| * |me| = X
// Post: |ret| = X

{ Bignat *v; // v will be the result. Using a pointer again for same reasons
  v = new Bignat (0); // Init |*v| = 0. We will use v for (*v) in the
                      // comments from now on

  deque <int>::const_iterator i;
  for (i = r.begin(); i != r.end(); i++)
      { 
        (*v).ten_times();     // v = 10 * r;
        Bignat t ( b );       // t = b;
        t.n_times ( *i );     // t = t * (*i) 
        (*v) = (*v) + t;      // v = v + t

      }
 return (*v);      
}

ostream &operator<< (ostream &os, const Bignat &b)

{
  ostream_iterator<int> out (os, "");
  copy ( b.r.begin(), b.r.end(), out);

}

istream &operator>> ( istream &is, Bignat &b)
{

 b.r.erase (b.r.begin(), b.r.end()); // Empty the deque

 char c;
 
 b.r.push_back ( 0 ); // To make sure b contains something
 while (c = is.get())
     { if (c==10) break; // 10 for eoln. Not very nice, but I couldn't find
                         // eoln in iostream
       b.r.push_back( (int) c - (int) '0'); // Pushing it in the deque
     } 
 
 


 // Keep the invariant. 

 // Remove leading zero's
 deque <int >::iterator i;
 i = b.r.begin();

  while (i+1 != b.r.end() && (*i) == 0 ) 
         {b.r.pop_front(); // Iterators no longer valid 
          i = b.r.begin();
         }
 
 // All digits between 0 and 9
 for (i = b.r.begin();i!= b.r.end();i++)
     (*i) = (*i) % 10;
 
 return is;
}


