-- File: dynamicstack.e
-- Author: Oscar Nierstrasz 27.11.95
-- An absolutely minimal stack implementation using linked lists.

-- NB: The operation names are NOT consistent with the Eiffel/S library

class DYNAMICSTACK [T]

-----------------------------------------------------------
creation { ANY }
	make

-----------------------------------------------------------
feature { NONE }
-----------------------------------------------------------
	topCell : MYCELL [T]
	size : INTEGER

	make is
		do
			size := 0
			-- NB: the initial cell is uninitialized
			-- and will cause an exception to be raised
			-- if it is dereferenced (i.e., when size = 0)
			!!topCell.make
		end -- make

-----------------------------------------------------------
feature { ANY }

	count : INTEGER is
		do
			Result := size
		end -- count

	empty : BOOLEAN is
		do
			Result := (size = 0)
		end -- empty

	push (x : T) is
		local
			newCell : MYCELL [T]
		do
			size := size + 1
			!!newCell.make
			newCell.setValue(deep_clone (x))
			newCell.setNext(topCell)
			topCell := newCell
		ensure
			not empty
			top = x
			size = old size + 1
		end -- push

	top : T is
		require
			not empty
		do
			Result := topCell.value
		end -- top

	pop is
		require
			not empty
		do
			-- must not be deferenced if stack is empty!
			topCell := topCell.next
			size := size - 1
		ensure
			size = old size - 1
		end -- pop

	invariant
		0 <= size

-----------------------------------------------------------
end -- class DYNAMICSTACK

