
// FILE: llist.cpp

#include "llist.h"

// constructor for an empty stack:
LList::LList (void)
	: size(0), frontCell(0), backCell(0)
{
}

// WARNING: since LList is not in OCF, there is no copy constructor
// defined.  If an instance of LList is passed by value instead of
// by reference, a shallow copy will be created.  When the shallow
// copy is destructed by LList::~LList, the original will also
// be damaged!

// destructor pops all cells:
LList::~LList(void)
{
	while (!this->empty()) {
		this->pop_front();
	}
}

void
LList::push_front(Item item)
{
	Cell *newCell;
	newCell = new Cell(item, this->frontCell); // i.e., even if frontCell == 0
	this->frontCell = newCell;
	if (this->empty()) {
		// no longer empty, so set backCell to point here too
		this->backCell = newCell;
	}
	size++;
}

Item
LList::front(void) throw(xmsg)
{
	if (this->empty()) {
		throw(xmsg("Empty list has no front!"));
	}
	return this->frontCell->value;
}

void
LList::pop_front(void) throw(xmsg)
{
	if (this->empty()) {
		throw(xmsg("Can't pop_front an empty list!"));
	}
	Cell *oldFront = frontCell;
	frontCell = frontCell->next;
	delete oldFront;
	size--;
	if (this->empty()) {
		// empty now, so forget the backCell pointer
		this->backCell = 0;
	}
}

// Builds a new backCell, 
void
LList::push_back(Item item)
{
	Cell *newCell;
	newCell = new Cell(item, 0);
	if (this->empty()) {
		this->frontCell = newCell;
	} else {
		this->backCell->next = newCell;
	}
	this->backCell = newCell;
	size++;
}

Item
LList::back(void) throw(xmsg)
{
	if (this->empty()) {
		throw(xmsg("Empty list has no back!"));
	}
	return this->backCell->value;
}

/*
LList::LList(const LList& arg)
{
	throw(xmsg("LList::LList(LList&) not implemented"));
}
*/

LList&
LList::operator=(const LList& arg)
{
	throw(xmsg("LList::operator=(LList&) not implemented"));
	return *this; // never get here
}

void
LList::print(ostream& os)
{
	os << "{";
	if (!this->empty()) {
		Cell *cell = this->frontCell;
		while (cell != 0) {
			os << " " << cell->value;
			cell = cell->next;
		}
	}
	os << " }";
}
