
/*
	File: TicTacToe.h
	Author: Oscar Nierstrasz 29.2.96
	
	Tic Tac Toe interface
	
	Invariants:
	- initially winner is nobody
	- game is over when all squares are occupied,
	  or there is a winner
	- winner is X or O iff winner has three in a row
	- turn is X or O
	- initially it is the turn of X
	- X and O swap turns (turn never equals previous turn)
	- a player can't mark a square that is already marked
*/


#ifndef TICTACTOE_H
#define TICTACTOE_H

#include <exception.h>

class TicTacToe {
	friend ostream& operator<<(ostream&, TicTacToe&);

  public :
	TicTacToe(void);		// constructor
					// NB: use default destructor
					// Don't need copy constructor
					// or assignment operator.
	
	int notover (void);		// true if game is not over
	const char *winner (void);	// Winner is "X", "O" or "Nobody"
	char turn (void);		// Whose turn is it?

	void play (int move)		// Current player marks a square
		throw(xmsg);		// Invalid move raises exception
	
	void print(void);		// Pretty-print the current state

  private :

	// Symbolic names for players -- also used to index char []
	enum Player  {
		nobody	= 0,
		X		= 1,
		O		= 2
	};
	
	// Private state:

	Player _winner;			// Initially nobody
	Player _turn;			// Initially X
	int squaresLeft;		// initially 9
	Player square[9];		// Initially nobody

	// Char and string names of players.
	// Must be initialized outside of class [ARM p. 290]
	static const char player[3];
	static const char * winners [3];
	
	// Private helper functions for play and print:
	
	char showSquare(int);		// Current ascii value of square
	void checkWinner(void);		// Is there a winner?
	int matchThree(int,int,int);	// Check for three in a row.
};


#endif // TICTACTOE_H
