
/*
	From C++ FAQs
	FAQ # 141: "How do you get around the hiding rule?"
*/

#include <iostream.h>

class Base
{
public:
	// virtual ~Base(void) {}
	virtual void f(float x) { cout << "Base::f(float)" << endl; }
	virtual void g(float x) { cout << "Base::g(float)" << endl; }
	virtual void h(float x) { cout << "Base::h(float)" << endl; }
};

class Derived : public Base
{
public:
	virtual void f(float x) { cout << "Derived::f(float)" << endl; }
	virtual void g(int x) { cout << "Derived::g(int)" << endl; }

	// redefine Base::g(float); Derived::g is now overloaded
	virtual void g(float x) {
		cout << "Derived::g(float)" << endl;
		Base::g(x);
	}

	// since Base::h(float) is virtual, this is now an override
	virtual void h(float x) { cout << "Derived::h(float)" << endl; }
};

int
main(void)
{
	Derived d;
	Base* bp = &d;
	Derived* dp = &d;

	// Good: behaviour depends solely on type of the object:
	bp->f(3.14f); // Calls Derived::f(float)
	dp->f(3.14f); // Calls Derived::f(float)

	bp->g(3.14f); // Calls Derived::g(float)
	dp->g(3.14f); // Calls Derived::g(float)

	bp->h(3.14f); // Calls Derived::h(float)
	dp->h(3.14f); // Calls Derived::h(float)

	return 0;
}

/*
	Output:
Derived::f(float)
Derived::f(float)
Derived::g(float)
Base::g(float)
Derived::g(float)
Base::g(float)
Derived::h(float)
Derived::h(float)
*/

