
/*
	From C++ FAQs
	FAQ # 136: "Is array-of Derived a kind-of array-of Base?"
	"NO! (although the compiler doesn't detect this error)."

	"[this problem" would have been properly trapped as an error if
	an Array<Derived> had been used rather than a Derived[] ...
	attempting to pass an Array<Derived> to f(Array<Base>& a) will
	cause a compile time error"

	"The fundamental problem is that a pointer to the first of an
	array-of things has exactly the same type as a pointer to a
	single thing.  This was inherited from C."
*/

#include <iostream.h>

#define BUGGY

class Base
{
public:
	Base(void) : i(77) { }
	virtual ~Base() { }
	virtual void service (void) { cout << "Base::service()" << endl; }
protected:
	int i;
};

class Derived : public Base
{
public:
	Derived(void) : Base(), j(88) { }
	virtual ~Derived() { }
	virtual void service (void) { cout << "Derived::service()" << endl; }
protected:
	int j;
};


#ifdef BUGGY
void useSubscript(Base* b)
{
	cout << "b[0].service(): " << flush;
	b[0].service();
	cout << "b[1].service(): " << flush;
	b[1].service(); // BOOM!
}

int
main(void)
{
	Derived d[10];
	useSubscript(d);
	return 0;
}

/*
	Output:
	b[0].service(): Derived::service()
	b[1].service(): Bus Error (core dumped)
*/

#else

#include <vector.h>

void useSubscript(vector<Base>& b)
{
	cout << "b[0].service(): " << flush;
	b[0].service();
	cout << "b[1].service(): " << flush;
	b[1].service(); // BOOM!
}

int
main(void)
{
	vector<Derived> d(10);
	useSubscript(d);
	return 0;
}

/*
subArray.C: In function `int main()':
subArray.C:74: cannot convert type `vector<Derived>' to type `vector<Base> &'
subArray.C:63: in passing argument 1 of `useSubscript(vector<Base> &)'
*** Error code 1
*/

#endif

